@@ -183,11 +183,18 @@ empathy_icon_name_for_presence (TpConnectionPresenceType presence)
 	case TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY:
 		return EMPATHY_IMAGE_EXT_AWAY;
 	case TP_CONNECTION_PRESENCE_TYPE_HIDDEN:
-		return EMPATHY_IMAGE_HIDDEN;
+		if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (),
+					     EMPATHY_IMAGE_HIDDEN))
+			return EMPATHY_IMAGE_HIDDEN;
+
+		/* The 'hidden' icon is not an official one so we fallback to offline if
+		 * it's not implemented */
+		return EMPATHY_IMAGE_OFFLINE;
 	case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
 	case TP_CONNECTION_PRESENCE_TYPE_ERROR:
-	case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
 		return EMPATHY_IMAGE_OFFLINE;
+	case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
+		return EMPATHY_IMAGE_PENDING;
 	case TP_CONNECTION_PRESENCE_TYPE_UNSET:
 		return NULL;
 	}
@@ -1385,10 +1392,10 @@ empathy_window_iconify (GtkWindow *window, GtkStatusIcon *status_icon)
 
 /* Takes care of moving the window to the current workspace. */
 void
-empathy_window_present (GtkWindow *window)
+empathy_window_present_with_time (GtkWindow *window,
+			guint32 timestamp)
 {
 	GdkWindow *gdk_window;
-	guint32 timestamp;
 
 	g_return_if_fail (GTK_IS_WINDOW (window));
 
@@ -1410,25 +1417,17 @@ empathy_window_present (GtkWindow *window)
 			gtk_widget_hide (GTK_WIDGET (window));
 	}
 
-	timestamp = gtk_get_current_event_time ();
-	if (timestamp == 0 && gdk_window != NULL) {
-		GdkEventMask mask;
-
-		/* According to the documentation of gdk_x11_get_server_time
-		 * GDK_PROPERTY_CHANGE_MASK needs to be set in its events otherwise a hang
-		 * can occur. Be sure to at least temporarily set this mask */
-		mask = gdk_window_get_events (gdk_window);
-		gdk_window_set_events (gdk_window,
-			mask | GDK_PROPERTY_CHANGE_MASK);
-		timestamp = gdk_x11_get_server_time (gdk_window);
-		gdk_window_set_events (gdk_window, mask);
-	}
-
 	gtk_window_present_with_time (window, timestamp);
 	gtk_window_set_skip_taskbar_hint (window, FALSE);
 	gtk_window_deiconify (window);
 }
 
+void
+empathy_window_present (GtkWindow *window)
+{
+  empathy_window_present_with_time (window, GDK_CURRENT_TIME);
+}
+
 GtkWindow *
 empathy_get_toplevel_window (GtkWidget *widget)
 {
