@@ -619,6 +619,8 @@ emit_error_signal (EmpathyFTHandler *handler,
 {
   EmpathyFTHandlerPriv *priv = GET_PRIV (handler);
 
+  DEBUG ("Error in transfer: %s\n", error->message);
+
   if (!g_cancellable_is_cancelled (priv->cancellable))
     g_cancellable_cancel (priv->cancellable);
 
@@ -755,7 +757,8 @@ ft_handler_push_to_dispatcher (EmpathyFTHandler *handler)
 
   /* I want to own a reference to the request, and destroy it later */
   empathy_dispatcher_create_channel (priv->dispatcher, connection,
-      g_hash_table_ref (priv->request), ft_handler_create_channel_cb, handler);
+      g_hash_table_ref (priv->request), EMPATHY_DISPATCHER_NON_USER_ACTION,
+      ft_handler_create_channel_cb, handler);
 }
 
 static void
@@ -820,6 +823,7 @@ hash_job_done (gpointer user_data)
   if (hash_data->error != NULL)
     {
       error = hash_data->error;
+      hash_data->error = NULL;
       goto cleanup;
     }
 
