@@ -182,6 +182,22 @@ debug_message_list_free (gpointer data)
   g_list_free (list);
 }
 
+static gchar *
+get_active_cm_name (EmpathyDebugWindow *self)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (self);
+  GtkTreeIter iter;
+  gchar *name;
+
+  if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter))
+    return NULL;
+
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
+      COL_CM_NAME, &name, -1);
+
+  return name;
+}
+
 static void
 debug_window_cache_new_message (EmpathyDebugWindow *debug_window,
     gdouble timestamp,
@@ -190,15 +206,11 @@ debug_window_cache_new_message (EmpathyDebugWindow *debug_window,
     const gchar *message)
 {
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
-  GtkTreeIter iter;
   GList *messages;
   DebugMessage *dm;
   char *name;
 
-  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter);
-
-  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
-      COL_CM_NAME, &name, -1);
+  name = get_active_cm_name (debug_window);
   messages = g_hash_table_lookup (priv->all_cms, name);
 
   dm = debug_message_new (timestamp, domain, level, message);
@@ -329,7 +341,6 @@ debug_window_get_messages_cb (TpProxy *proxy,
 {
   EmpathyDebugWindow *debug_window = (EmpathyDebugWindow *) user_data;
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
-  GtkTreeIter iter;
   gchar *name;
   GList *old_messages;
   guint i;
@@ -343,10 +354,7 @@ debug_window_get_messages_cb (TpProxy *proxy,
 
   debug_window_set_toolbar_sensitivity (debug_window, TRUE);
 
-  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter);
-
-  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
-      COL_CM_NAME, &name, -1);
+  name = get_active_cm_name (debug_window);
   old_messages = g_hash_table_lookup (priv->all_cms, name);
 
   /* we call get_messages either when a new CM is added or
@@ -768,12 +776,12 @@ debug_window_fill_cm_chooser (EmpathyDebugWindow *debug_window)
 }
 
 static void
-debug_window_pause_toggled_cb (GtkToggleToolButton *pause_,
+debug_window_pause_toggled_cb (GtkToggleToolButton *pause,
     EmpathyDebugWindow *debug_window)
 {
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
 
-  priv->paused = gtk_toggle_tool_button_get_active (pause_);
+  priv->paused = gtk_toggle_tool_button_get_active (pause);
 
   debug_window_set_enabled (debug_window, !priv->paused);
 }
@@ -1062,6 +1070,10 @@ debug_window_save_clicked_cb (GtkToolButton *tool_button,
     EmpathyDebugWindow *debug_window)
 {
   GtkWidget *file_chooser;
+  gchar *name, *tmp = NULL;
+  char time_str[32];
+  time_t t;
+  struct tm *tm_s;
 
   file_chooser = gtk_file_chooser_dialog_new (_("Save"),
       GTK_WINDOW (debug_window), GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -1073,6 +1085,26 @@ debug_window_save_clicked_cb (GtkToolButton *tool_button,
   gtk_file_chooser_set_do_overwrite_confirmation (
       GTK_FILE_CHOOSER (file_chooser), TRUE);
 
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser),
+      g_get_tmp_dir ());
+
+  name = get_active_cm_name (debug_window);
+
+  t = time (NULL);
+  tm_s = localtime (&t);
+  if (tm_s != NULL)
+    {
+      if (strftime(time_str, sizeof (time_str), "%d-%m-%y_%H-%M-%S", tm_s))
+        tmp = g_strdup_printf ("%s-%s.log", name, time_str);
+    }
+
+  if (tmp == NULL)
+    tmp = g_strdup_printf ("%s.log", name);
+  g_free (name);
+
+  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_chooser), tmp);
+  g_free (tmp);
+
   g_signal_connect (file_chooser, "response",
       G_CALLBACK (debug_window_save_file_chooser_response_cb),
       debug_window);
