@@ -25,6 +25,7 @@
 
 #include <sys/stat.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 
 #include <telepathy-glib/account-manager.h>
@@ -53,8 +54,8 @@
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include "empathy-accounts-dialog.h"
+#include "empathy-chat-manager.h"
 #include "empathy-main-window.h"
-#include "ephy-spinner.h"
 #include "empathy-preferences.h"
 #include "empathy-about-dialog.h"
 #include "empathy-debug-window.h"
@@ -92,6 +93,7 @@ typedef struct {
 	GtkWidget              *window;
 	GtkWidget              *main_vbox;
 	GtkWidget              *throbber;
+	GtkWidget              *throbber_tool_item;
 	GtkWidget              *presence_toolbar;
 	GtkWidget              *presence_chooser;
 	GtkWidget              *errors_vbox;
@@ -506,9 +508,11 @@ main_window_update_status (EmpathyMainWindow *window)
 
 	/* Update the spinner state */
 	if (connecting) {
-		ephy_spinner_start (EPHY_SPINNER (window->throbber));
+		gtk_spinner_start (GTK_SPINNER (window->throbber));
+		gtk_widget_show (window->throbber_tool_item);
 	} else {
-		ephy_spinner_stop (EPHY_SPINNER (window->throbber));
+		gtk_spinner_stop (GTK_SPINNER (window->throbber));
+		gtk_widget_hide (window->throbber_tool_item);
 	}
 
 	/* Update widgets sensibility */
@@ -662,6 +666,23 @@ main_window_destroy_cb (GtkWidget         *widget,
 	g_free (window);
 }
 
+static gboolean
+main_window_key_press_event_cb  (GtkWidget *window,
+				 GdkEventKey *event,
+				 gpointer user_data)
+{
+	EmpathyChatManager *chat_manager;
+
+	if (event->keyval == GDK_T
+	    && event->state & GDK_SHIFT_MASK
+	    && event->state & GDK_CONTROL_MASK) {
+		chat_manager = empathy_chat_manager_dup_singleton ();
+		empathy_chat_manager_undo_closed_chat (chat_manager);
+		g_object_unref (chat_manager);
+	}
+	return FALSE;
+}
+
 static void
 main_window_chat_quit_cb (GtkAction         *action,
 			  EmpathyMainWindow *window)
@@ -712,7 +733,7 @@ main_window_view_show_offline_cb (GtkToggleAction   *action,
 
 	current = gtk_toggle_action_get_active (action);
 	empathy_conf_set_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_CONTACTS_SHOW_OFFLINE,
+			      EMPATHY_PREFS_UI_SHOW_OFFLINE,
 			      current);
 
 	/* Turn off sound just while we alter the contact list. */
@@ -887,7 +908,8 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 
 	if (connection != NULL) {
 		DEBUG ("Requesting channel for '%s'", room);
-		empathy_dispatcher_join_muc (connection, room, NULL, NULL);
+		empathy_dispatcher_join_muc (connection, room,
+			gtk_get_current_event_time (), NULL, NULL);
 	}
 }
 
@@ -1136,7 +1158,7 @@ main_window_help_contents_cb (GtkAction         *action,
 }
 
 static gboolean
-main_window_throbber_button_press_event_cb (GtkWidget         *throbber_ebox,
+main_window_throbber_button_press_event_cb (GtkWidget         *throbber,
 					    GdkEventButton    *event,
 					    EmpathyMainWindow *window)
 {
@@ -1146,7 +1168,7 @@ main_window_throbber_button_press_event_cb (GtkWidget         *throbber_ebox,
 	}
 
 	empathy_accounts_dialog_show_application (
-			gtk_widget_get_screen (GTK_WIDGET (throbber_ebox)),
+			gtk_widget_get_screen (GTK_WIDGET (throbber)),
 			NULL, FALSE, FALSE);
 
 	return FALSE;
@@ -1299,7 +1321,6 @@ empathy_main_window_show (void)
 	EmpathyConf              *conf;
 	GtkWidget                *sw;
 	GtkToggleAction          *show_offline_widget;
-	GtkWidget                *ebox;
 	GtkAction                *show_map_widget;
 	GtkToolItem              *item;
 	gboolean                  show_offline;
@@ -1338,6 +1359,7 @@ empathy_main_window_show (void)
 
 	empathy_builder_connect (gui, window,
 			      "main_window", "destroy", main_window_destroy_cb,
+			      "main_window", "key-press-event", main_window_key_press_event_cb,
 			      "chat_quit", "activate", main_window_chat_quit_cb,
 			      "chat_new_message", "activate", main_window_chat_new_message_cb,
 			      "chat_new_call", "activate", main_window_chat_new_call_cb,
@@ -1410,24 +1432,21 @@ empathy_main_window_show (void)
 	gtk_toolbar_insert (GTK_TOOLBAR (window->presence_toolbar), item, -1);
 
 	/* Set up the throbber */
-	ebox = gtk_event_box_new ();
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
-	gtk_widget_set_tooltip_text (ebox, _("Show and edit accounts"));
-	g_signal_connect (ebox,
-			  "button-press-event",
-			  G_CALLBACK (main_window_throbber_button_press_event_cb),
-			  window);
-	gtk_widget_show (ebox);
-
-	window->throbber = ephy_spinner_new ();
-	ephy_spinner_set_size (EPHY_SPINNER (window->throbber), GTK_ICON_SIZE_LARGE_TOOLBAR);
-	gtk_container_add (GTK_CONTAINER (ebox), window->throbber);
+	window->throbber = gtk_spinner_new ();
+	gtk_widget_set_size_request (window->throbber, 16, -1);
+	gtk_widget_set_tooltip_text (window->throbber, _("Show and edit accounts"));
+	gtk_widget_set_has_window (GTK_WIDGET (window->throbber), TRUE);
+	gtk_widget_set_events (window->throbber, GDK_BUTTON_PRESS_MASK);
+	g_signal_connect (window->throbber, "button-press-event",
+		G_CALLBACK (main_window_throbber_button_press_event_cb),
+		window);
 	gtk_widget_show (window->throbber);
 
 	item = gtk_tool_item_new ();
-	gtk_container_add (GTK_CONTAINER (item), ebox);
+	gtk_container_set_border_width (GTK_CONTAINER (item), 6);
 	gtk_toolbar_insert (GTK_TOOLBAR (window->presence_toolbar), item, -1);
-	gtk_widget_show (GTK_WIDGET (item));
+	gtk_container_add (GTK_CONTAINER (item), window->throbber);
+	window->throbber_tool_item = GTK_WIDGET (item);
 
 	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
 	monitor = empathy_contact_list_get_monitor (list_iface);
@@ -1484,10 +1503,10 @@ empathy_main_window_show (void)
 
 	/* Show offline ? */
 	empathy_conf_get_bool (conf,
-			      EMPATHY_PREFS_CONTACTS_SHOW_OFFLINE,
+			      EMPATHY_PREFS_UI_SHOW_OFFLINE,
 			      &show_offline);
 	empathy_conf_notify_add (conf,
-				EMPATHY_PREFS_CONTACTS_SHOW_OFFLINE,
+				EMPATHY_PREFS_UI_SHOW_OFFLINE,
 				main_window_notify_show_offline_cb,
 				show_offline_widget);
 
