@@ -41,13 +41,12 @@
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include "empathy-map-view.h"
-#include "ephy-spinner.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_LOCATION
 #include <libempathy/empathy-debug.h>
 
 typedef struct {
-  EmpathyContactListStore *list_store;
+  EmpathyContactList *contact_list;
 
   GtkWidget *window;
   GtkWidget *zoom_in;
@@ -56,6 +55,9 @@ typedef struct {
   ChamplainView *map_view;
   ChamplainLayer *layer;
   guint timeout_id;
+  /* reffed (EmpathyContact *) => borrowed (ChamplainMarker *) */
+  GHashTable *markers;
+  gulong members_changed_id;
 } EmpathyMapView;
 
 static void
@@ -67,28 +69,61 @@ map_view_state_changed (ChamplainView *view,
 
   g_object_get (G_OBJECT (view), "state", &state, NULL);
   if (state == CHAMPLAIN_STATE_LOADING)
-    ephy_spinner_start (EPHY_SPINNER (window->throbber));
+    {
+      gtk_spinner_start (GTK_SPINNER (window->throbber));
+      gtk_widget_show (window->throbber);
+    }
   else
-    ephy_spinner_stop (EPHY_SPINNER (window->throbber));
+    {
+      gtk_spinner_stop (GTK_SPINNER (window->throbber));
+      gtk_widget_hide (window->throbber);
+    }
 }
 
+static gboolean
+contact_has_location (EmpathyContact *contact)
+{
+  GHashTable *location;
+
+  location = empathy_contact_get_location (contact);
+
+  if (location == NULL || g_hash_table_size (location) == 0)
+    return FALSE;
+
+  return TRUE;
+}
+
+static ChamplainMarker * create_marker (EmpathyMapView *window,
+    EmpathyContact *contact);
+
 static void
-map_view_marker_update_position (ChamplainMarker *marker,
+map_view_update_contact_position (EmpathyMapView *self,
     EmpathyContact *contact)
 {
   gdouble lon, lat;
   GValue *value;
   GHashTable *location;
+  ChamplainMarker *marker;
+  gboolean has_location;
+
+  has_location = contact_has_location (contact);
+
+  marker = g_hash_table_lookup (self->markers, contact);
+  if (marker == NULL)
+    {
+      if (!has_location)
+        return;
+
+      marker = create_marker (self, contact);
+    }
+  else if (!has_location)
+    {
+      champlain_base_marker_animate_out (CHAMPLAIN_BASE_MARKER (marker));
+      return;
+    }
 
   location = empathy_contact_get_location (contact);
 
-  if (location == NULL ||
-      g_hash_table_size (location) == 0)
-  {
-    champlain_base_marker_animate_out (CHAMPLAIN_BASE_MARKER (marker));
-    return;
-  }
-
   value = g_hash_table_lookup (location, EMPATHY_LOCATION_LAT);
   if (value == NULL)
     {
@@ -112,9 +147,9 @@ map_view_marker_update_position (ChamplainMarker *marker,
 static void
 map_view_contact_location_notify (EmpathyContact *contact,
     GParamSpec *arg1,
-    ChamplainMarker *marker)
+    EmpathyMapView *self)
 {
-  map_view_marker_update_position (marker, contact);
+  map_view_update_contact_position (self, contact);
 }
 
 static void
@@ -223,30 +258,14 @@ map_view_contacts_update_label (ChamplainMarker *marker)
   g_free (label);
 }
 
-static gboolean
-map_view_contacts_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    gpointer user_data)
+static ChamplainMarker *
+create_marker (EmpathyMapView *window,
+    EmpathyContact *contact)
 {
-  EmpathyMapView *window = (EmpathyMapView *) user_data;
-  EmpathyContact *contact;
   ClutterActor *marker;
   ClutterActor *texture;
-  GHashTable *location;
   GdkPixbuf *avatar;
 
-  gtk_tree_model_get (model, iter, EMPATHY_CONTACT_LIST_STORE_COL_CONTACT,
-     &contact, -1);
-
-  if (contact == NULL)
-    return FALSE;
-
-  location = empathy_contact_get_location (contact);
-
-  if (location == NULL || g_hash_table_size (location) == 0)
-    return FALSE;
-
   marker = champlain_marker_new ();
 
   avatar = empathy_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
@@ -264,6 +283,8 @@ map_view_contacts_foreach (GtkTreeModel *model,
   g_object_set_data_full (G_OBJECT (marker), "contact",
       g_object_ref (contact), g_object_unref);
 
+  g_hash_table_insert (window->markers, g_object_ref (contact), marker);
+
   map_view_contacts_update_label (CHAMPLAIN_MARKER (marker));
 
   clutter_actor_set_reactive (CLUTTER_ACTOR (marker), TRUE);
@@ -272,38 +293,38 @@ map_view_contacts_foreach (GtkTreeModel *model,
 
   clutter_container_add (CLUTTER_CONTAINER (window->layer), marker, NULL);
 
+  return CHAMPLAIN_MARKER (marker);
+}
+
+static void
+contact_added (EmpathyMapView *window,
+    EmpathyContact *contact)
+{
   g_signal_connect (contact, "notify::location",
-      G_CALLBACK (map_view_contact_location_notify), marker);
+      G_CALLBACK (map_view_contact_location_notify), window);
 
-  map_view_marker_update_position (CHAMPLAIN_MARKER (marker), contact);
-
-  g_object_unref (contact);
-  return FALSE;
+  map_view_update_contact_position (window, contact);
 }
 
 static void
 map_view_destroy_cb (GtkWidget *widget,
     EmpathyMapView *window)
 {
-  GList *item;
+  GHashTableIter iter;
+  gpointer contact;
 
   g_source_remove (window->timeout_id);
 
-  item = clutter_container_get_children (CLUTTER_CONTAINER (window->layer));
-  while (item != NULL)
-  {
-    EmpathyContact *contact;
-    ChamplainMarker *marker;
-
-    marker = CHAMPLAIN_MARKER (item->data);
-    contact = g_object_get_data (G_OBJECT (marker), "contact");
+  g_hash_table_iter_init (&iter, window->markers);
+  while (g_hash_table_iter_next (&iter, &contact, NULL))
     g_signal_handlers_disconnect_by_func (contact,
-        map_view_contact_location_notify, marker);
+        map_view_contact_location_notify, window);
 
-    item = g_list_next (item);
-  }
+  g_signal_handler_disconnect (window->contact_list,
+      window->members_changed_id);
 
-  g_object_unref (window->list_store);
+  g_hash_table_destroy (window->markers);
+  g_object_unref (window->contact_list);
   g_object_unref (window->layer);
   g_slice_free (EmpathyMapView, window);
 }
@@ -336,6 +357,39 @@ map_view_tick (EmpathyMapView *window)
   return TRUE;
 }
 
+static void
+contact_removed (EmpathyMapView *self,
+    EmpathyContact *contact)
+{
+  ClutterActor *marker;
+
+  marker = g_hash_table_lookup (self->markers, contact);
+  if (marker == NULL)
+    return;
+
+  clutter_actor_destroy (marker);
+  g_hash_table_remove (self->markers, contact);
+}
+
+static void
+members_changed_cb (EmpathyContactList *list,
+    EmpathyContact *contact,
+    EmpathyContact *actor,
+    guint reason,
+    gchar *message,
+    gboolean is_member,
+    EmpathyMapView *self)
+{
+  if (is_member)
+    {
+      contact_added (self, contact);
+    }
+  else
+    {
+      contact_removed (self, contact);
+    }
+}
+
 GtkWidget *
 empathy_map_view_show (void)
 {
@@ -345,9 +399,7 @@ empathy_map_view_show (void)
   GtkWidget *embed;
   GtkWidget *throbber_holder;
   gchar *filename;
-  GtkTreeModel *model;
-  EmpathyContactList *list_iface;
-  EmpathyContactListStore *list_store;
+  GList *members, *l;
 
   if (window)
     {
@@ -381,20 +433,16 @@ empathy_map_view_show (void)
   /* Clear the static pointer to window if the dialog is destroyed */
   g_object_add_weak_pointer (G_OBJECT (window->window), (gpointer *) &window);
 
-  list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
-  list_store = empathy_contact_list_store_new (list_iface);
-  empathy_contact_list_store_set_show_groups (list_store, FALSE);
-  empathy_contact_list_store_set_show_avatars (list_store, TRUE);
-  g_object_unref (list_iface);
+  window->contact_list = EMPATHY_CONTACT_LIST (
+      empathy_contact_manager_dup_singleton ());
 
-  window->throbber = ephy_spinner_new ();
-  ephy_spinner_set_size (EPHY_SPINNER (window->throbber),
-      GTK_ICON_SIZE_LARGE_TOOLBAR);
-  gtk_widget_show (window->throbber);
+  window->members_changed_id = g_signal_connect (window->contact_list,
+      "members-changed", G_CALLBACK (members_changed_cb), window);
+
+  window->throbber = gtk_spinner_new ();
+  gtk_widget_set_size_request (window->throbber, 16, 16);
   gtk_container_add (GTK_CONTAINER (throbber_holder), window->throbber);
 
-  window->list_store = list_store;
-
   /* Set up map view */
   embed = gtk_champlain_embed_new ();
   window->map_view = gtk_champlain_embed_get_view (GTK_CHAMPLAIN_EMBED (embed));
@@ -412,8 +460,17 @@ empathy_map_view_show (void)
       G_CALLBACK (map_view_state_changed), window);
 
   /* Set up contact list. */
-  model = GTK_TREE_MODEL (window->list_store);
-  gtk_tree_model_foreach (model, map_view_contacts_foreach, window);
+  window->markers = g_hash_table_new_full (NULL, NULL,
+      (GDestroyNotify) g_object_unref, NULL);
+
+  members = empathy_contact_list_get_members (
+      window->contact_list);
+  for (l = members; l != NULL; l = g_list_next (l))
+    {
+      contact_added (window, l->data);
+      g_object_unref (l->data);
+    }
+  g_list_free (members);
 
   empathy_window_present (GTK_WINDOW (window->window));
 
@@ -423,4 +480,3 @@ empathy_map_view_show (void)
 
   return window->window;
 }
-
