@@ -24,6 +24,17 @@ test_replace_match (const gchar *text,
   g_string_append_c (string, ']');
 }
 
+static void
+test_replace_verbatim (const gchar *text,
+                       gssize len,
+                       gpointer match_data,
+                       gpointer user_data)
+{
+  GString *string = user_data;
+
+  g_string_append_len (string, text, len);
+}
+
 static void
 test_parsers (void)
 {
@@ -59,29 +70,25 @@ test_parsers (void)
       "Foo (www.foo.com)", "Foo ([www.foo.com])",
       "Foo {www.foo.com}", "Foo {[www.foo.com]}",
       "Foo [www.foo.com]", "Foo [[www.foo.com]]",
-      "Foo <www.foo.com>", "Foo &lt;[www.foo.com]&gt;",
-      "Foo \"www.foo.com\"", "Foo &quot;[www.foo.com]&quot;",
+      "Foo <www.foo.com>", "Foo <[www.foo.com]>",
+      "Foo \"www.foo.com\"", "Foo \"[www.foo.com]\"",
       "Foo (www.foo.com/bar(123)baz)", "Foo ([www.foo.com/bar(123)baz])",
-      "<a href=\"http://foo.com\">bar</a>", "&lt;a href=&quot;[http://foo.com]&quot;&gt;bar&lt;/a&gt;",
+      "<a href=\"http://foo.com\">bar</a>", "<a href=\"[http://foo.com]\">bar</a>",
       "Foo (user@server.com)", "Foo ([user@server.com])",
       "Foo {user@server.com}", "Foo {[user@server.com]}",
       "Foo [user@server.com]", "Foo [[user@server.com]]",
-      "Foo <user@server.com>", "Foo &lt;[user@server.com]&gt;",
-      "Foo \"user@server.com\"", "Foo &quot;[user@server.com]&quot;",
+      "Foo <user@server.com>", "Foo <[user@server.com]>",
+      "Foo \"user@server.com\"", "Foo \"[user@server.com]\"",
 
       /* Basic smileys */
       "a:)b", "a[:)]b",
       ">:)", "[>:)]",
-      ">:(", "&gt;[:(]",
+      ">:(", ">[:(]",
 
       /* Smileys and links mixed */
       ":)http://foo.com", "[:)][http://foo.com]",
       "a :) b http://foo.com c :( d www.test.com e", "a [:)] b [http://foo.com] c [:(] d [www.test.com] e",
 
-      /* '\r' should be stripped */
-      "badger\n\rmushroom", "badger\nmushroom",
-      "badger\r\nmushroom", "badger\nmushroom",
-
       /* FIXME: Known issue: Brackets should be counted by the parser */
       //"Foo www.bar.com/test(123)", "Foo [www.bar.com/test(123)]",
       //"Foo (www.bar.com/test(123))", "Foo ([www.bar.com/test(123)])",
@@ -95,10 +102,11 @@ test_parsers (void)
     {
       {empathy_string_match_link, test_replace_match},
       {empathy_string_match_smiley, test_replace_match},
-      {empathy_string_match_all, empathy_string_replace_escaped},
+      {empathy_string_match_all, test_replace_verbatim},
       {NULL, NULL}
     };
   guint i;
+  gboolean failed = FALSE;
 
   DEBUG ("Started");
   for (i = 0; tests[i] != NULL; i += 2)
@@ -111,10 +119,13 @@ test_parsers (void)
 
       ok = !tp_strdiff (tests[i + 1], string->str);
       DEBUG ("'%s' => '%s': %s", tests[i], string->str, ok ? "OK" : "FAILED");
-      g_assert (ok);
+      if (!ok)
+        failed = TRUE;
 
       g_string_free (string, TRUE);
     }
+
+  g_assert (!failed);
 }
 
 int
